 /*
  * Copyright (C), 2018-2021
  * Project: starter
  * FileName: TokenLogin
  * Author:   Louis
  * Date:     2021/3/30 22:09
  */
 package com.fr.plugin.sso.request;

 import com.fanruan.api.decision.login.LoginKit;
import com.fanruan.api.log.LogKit;
import com.fanruan.api.net.NetworkKit;
import com.fanruan.api.util.CodeKit;
import com.fanruan.api.util.StringKit;
import com.fr.base.TemplateUtils;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.utils.DecisionServiceConstants;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.json.JSONObject;
import com.fr.plugin.sso.config.SsoConfig;
import com.fr.plugin.sso.service.SsoService;
import com.fr.stable.CodeUtils;
import com.fr.stable.StringUtils;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

 /**
  * <Function Description><br>
  * <TokenLogin>
  *
  * @author fr.open
  * @since 1.0.0
  */
 public class TokenLogin extends AbstractGlobalRequestFilterProvider {
     public static final String TPL_PATH = "/com/fr/plugin/sso/web/codePage.html";
     public static final String REMOTE_DESIGN = "/remote/design";
     public static final String RESOURCES_PATH = "/resources";
     public static final String FILE_PATH = "/file";
     public static final String SYSTEM_INFO = "/system/info";
     public static final String SYSTEM_HEALTH = "/system/health";
     public static final String MATERIALS_MIN_JS_MAP = "/materials.min.js.map";
     public static final String LOGIN_PATH = "/login";
     public static final String LOGIN_OTHER = "/login/";
     public static final String LOGOUT_PATH = "/logout";
     public static final String USER_LANGUAGE = "/v10/user/language";

     public static final String SSO_CALLBACK = "/sso_callback";
     public static final String CALLBACK = "callback";
     public static final String SOURCE_URL = "sourceUrl";

     private SsoService ssoService;

     /**
      * 过滤器名称
      *
      * @return
      */
     @Override
     public String filterName() {
         return "SsoFilter";
     }

     /**
      * 过滤规则
      *
      * @return
      */
     @Override
     public String[] urlPatterns() {
         return new String[]{"/*"};
     }

     /**
      * 过滤器初始化
      *
      * @param filterConfig
      */
     @Override
     public void init(FilterConfig filterConfig) {
         ssoService = new SsoService();
         super.init(filterConfig);
     }

     /**
      * 过滤器处理
      *
      * @param request
      * @param response
      * @param filterChain
      */
     @Override
     public void doFilter(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) {
         try {
             if (operation(request, response)) {
                 filterChain.doFilter(request, response);
             }
         } catch (Exception e) {
             LogKit.error(e.getMessage(), e);
         }
     }

     /**
      * 用户验证登陆操作
      *
      * @param req
      * @param res
      * @throws Exception
      */
     private boolean operation(HttpServletRequest req, HttpServletResponse res) throws Exception {
         String pathInfo = (req.getPathInfo() != null) ? req.getPathInfo() : StringKit.EMPTY;
         LogKit.info("sso-TokenLogin-operation-pathInfo:{}", pathInfo);
         if (pathInfo.startsWith(REMOTE_DESIGN) || pathInfo.startsWith(LOGIN_OTHER)
                 || StringKit.equals(LOGIN_PATH, pathInfo) || pathInfo.startsWith(SYSTEM_HEALTH)
                 || pathInfo.startsWith(RESOURCES_PATH) || pathInfo.startsWith(LOGOUT_PATH)
                 || pathInfo.startsWith(SYSTEM_INFO) || pathInfo.startsWith(MATERIALS_MIN_JS_MAP)
                 || pathInfo.startsWith(USER_LANGUAGE) || pathInfo.startsWith(FILE_PATH)) {
             return true;
         }
         // 已登录
         if (LoginService.getInstance().isLogged(req)) {
             return true;
         }
         String callback = NetworkKit.getHTTPRequestParameter(req, CALLBACK);
         if (StringKit.isBlank(callback)) {
             loginPage(req, res);
             return false;
         }
         if (!pathInfo.startsWith(SSO_CALLBACK)) {
             return true;
         }
         String ip = ssoService.getIpAddr(req);
         JSONObject ssoToken = ssoService.getSsoToken(callback, ip);
         LogKit.error("sso-TokenLogin-operation-ssoToken:{}", ssoToken);
         if (ssoToken == null || !StringKit.equals(ssoToken.getString("Success"), "true")) {
             return true;
         }
         //获取单点登陆用户的相关信息
         JSONObject ssoUserInfo = ssoService.getSsoUserInfo(ssoToken.getString("Message"), ip);
         if (ssoUserInfo == null || !StringKit.equals(ssoUserInfo.getString("Success"), "true")) {
             return true;
         }
         //往session设置ssoTokenMessage参数值
         req.getSession().setAttribute("ssoTokenMessage", ssoToken.get("Message").toString());
         //往session设置ssoUserInfo参数值
         req.getSession().setAttribute("ssoUserInfo", ssoUserInfo);
         String username = ssoUserInfo.get("AppAccount").toString();
         if (StringKit.isBlank(username)) {
             return true;
         }
         LogKit.error("sso-TokenLogin-operation-username:{}", username);
         String tokenFR = LoginKit.login(req, res, username);
         req.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, tokenFR);
         String sourceUrl = encodeViewlet(NetworkKit.getHTTPRequestParameter(req, SOURCE_URL));
         LogKit.error("sso-TokenLogin-operation-sourceUrl:{}", sourceUrl);
         res.sendRedirect(sourceUrl);
         return false;
     }

     /**
      * 登陆页面
      *
      * @param req
      * @param res
      * @return
      * @throws Exception
      */
     private void loginPage(HttpServletRequest req, HttpServletResponse res) throws Exception {
         Map<String, String> parameterMap = new HashMap<>();
         parameterMap.put("appId", SsoConfig.getInstance().getAppId());
         parameterMap.put("sourceUrl", CodeKit.encodeURIComponent(getOriginalURL(req)));
         String codePage = TemplateUtils.renderTemplate(TPL_PATH, parameterMap);
         LogKit.info("sso-TokenLogin-loginPage-codePage:{}", codePage);
         WebUtils.printAsString(res, codePage);
     }

     /**
      * 得到请求url和参数
      *
      * @param request
      * @return
      */
     private String getOriginalURL(HttpServletRequest request) {
         StringBuffer url = request.getRequestURL();
         if (StringKit.isNotBlank(request.getQueryString())) {
             url.append("?").append(request.getQueryString());
         }
         return url.toString();
     }

     /**
      * viewlet中文处理编码
      *
      * @param url
      * @return
      */
     private String encodeViewlet(String url) {
         if (StringKit.isBlank(url)) {
             return url;
         }
         String paramViewlet = StringUtils.EMPTY;
         if (url.contains("viewlet=")) {
             paramViewlet = url.split("viewlet=")[1];
             paramViewlet = paramViewlet.split("&")[0];
         }
         if (StringUtils.isNotEmpty(paramViewlet)) {
             url = url.replaceFirst(paramViewlet, CodeUtils.encodeURIComponent(paramViewlet));
         }
         return url;
     }
 }